import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { CandidateProps, CreateCandidateProps } from './candidate.types';
import { WayStepProps } from './value-objects/waystep.value-object';

export class CandidateEntity extends AggregateRoot<CandidateProps> {
  protected readonly _id: AggregateID;

  static create = (create: CreateCandidateProps): CandidateEntity => {
    const props: CandidateProps = { ...create };
    return new CandidateEntity({ id: create.id, props });
  };

  setCarpoolPath = (waySteps: WayStepProps[]): CandidateEntity => {
    this.props.carpoolSteps = waySteps;
    return this;
  };

  setMetrics = (distance: number, duration: number): CandidateEntity => {
    this.props.distance = distance;
    this.props.duration = duration;
    return this;
  };

  isDetourValid = (): boolean =>
    this._validateDistanceDetour() && this._validateDurationDetour();

  private _validateDurationDetour = (): boolean =>
    this.props.duration
      ? this.props.duration <=
        this.props.driverDuration *
          (1 + this.props.spacetimeDetourRatio.maxDurationDetourRatio)
      : false;

  private _validateDistanceDetour = (): boolean =>
    this.props.distance
      ? this.props.distance <=
        this.props.driverDistance *
          (1 + this.props.spacetimeDetourRatio.maxDistanceDetourRatio)
      : false;

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
