import { Role } from './ad.types';
import { PointProps } from './value-objects/point.value-object';
import { WayStepProps } from './value-objects/waystep.value-object';

// All properties that a Candidate has
export interface CandidateProps {
  role: Role;
  driverWaypoints: PointProps[];
  passengerWaypoints: PointProps[];
  driverDistance: number;
  driverDuration: number;
  carpoolSteps?: WayStepProps[]; // carpool path for the crew (driver + passenger)
  distance?: number;
  duration?: number;
  spacetimeDetourRatio: SpacetimeDetourRatio;
}

// Properties that are needed for a Candidate creation
export interface CreateCandidateProps {
  id: string;
  role: Role;
  driverDistance: number;
  driverDuration: number;
  driverWaypoints: PointProps[];
  passengerWaypoints: PointProps[];
  spacetimeDetourRatio: SpacetimeDetourRatio;
}

export enum Target {
  START = 'START',
  INTERMEDIATE = 'INTERMEDIATE',
  FINISH = 'FINISH',
  NEUTRAL = 'NEUTRAL',
}

export abstract class Validator {
  abstract validate(): boolean;
}

export type SpacetimeMetric = {
  distance: number;
  duration: number;
};

export type SpacetimeDetourRatio = {
  maxDistanceDetourRatio: number;
  maxDurationDetourRatio: number;
};
