import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface PointProps {
  lon: number;
  lat: number;
}

export class Point extends ValueObject<PointProps> {
  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  isSame = (point: this): boolean =>
    point.lon == this.lon && point.lat == this.lat;

  protected validate(props: PointProps): void {
    if (props.lon > 180 || props.lon < -180)
      throw new ArgumentOutOfRangeException('lon must be between -180 and 180');
    if (props.lat > 90 || props.lat < -90)
      throw new ArgumentOutOfRangeException('lat must be between -90 and 90');
  }
}
